"""class for the radio settings"""


from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QComboBox, QDialogButtonBox, QLineEdit

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
import meshtastic.radioconfig_pb2
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref
from meshtastic_flasher.util import zero_if_blank


class RadioForm(QDialog):
    """position settings form"""

    def __init__(self, parent=None):
        """constructor"""
        super(RadioForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Radio Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets
        self.is_router = QCheckBox()
        self.is_router.setToolTip(self.parent.description('is_router'))
        self.region = QComboBox()
        self.region.setToolTip(self.parent.description('region'))
        self.region.setMinimumContentsLength(17)
        self.debug_log_enabled = QCheckBox()
        self.debug_log_enabled.setToolTip(self.parent.description('debug_log_enabled'))
        self.serial_disabled = QCheckBox()
        self.serial_disabled.setToolTip(self.parent.description('serial_disabled'))
        self.auto_screen_carousel_secs = QLineEdit()
        self.auto_screen_carousel_secs.setToolTip(self.parent.description('auto_screen_carousel_secs'))
        self.frequency_offset = QLineEdit()
        self.frequency_offset.setToolTip(self.parent.description('frequency_offset'))
        self.hop_limit = QLineEdit()
        # TODO: hops_reliable?
        self.hop_limit.setToolTip(self.parent.description('hop_limit'))
        # TODO: see https://github.com/meshtastic/Meshtastic-python/issues/280 (add when fixed)
        #self.ignore_incoming = QLineEdit()
        self.is_lora_tx_disabled = QCheckBox()
        self.is_lora_tx_disabled.setToolTip(self.parent.description('is_lora_tx_disabled'))
        self.send_owner_interval = QLineEdit()
        self.send_owner_interval.setToolTip(self.parent.description('send_owner_interval'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)


        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.label("is_router"), self.is_router)
        form_layout.addRow(self.parent.label("region"), self.region)
        form_layout.addRow(self.parent.label("debug_log_enabled"), self.debug_log_enabled)
        form_layout.addRow(self.parent.label("serial_disabled"), self.serial_disabled)
        form_layout.addRow(self.parent.label("auto_screen_carousel_secs"), self.auto_screen_carousel_secs)
        form_layout.addRow(self.parent.label("frequency_offset"), self.frequency_offset)
        form_layout.addRow(self.parent.label("hop_limit"), self.hop_limit)
        #form_layout.addRow(self.parent.label("Ignore Incoming"), self.ignore_incoming)
        form_layout.addRow(self.parent.label("is_lora_tx_disabled"), self.is_lora_tx_disabled)
        form_layout.addRow(self.parent.label("send_owner_interval"), self.send_owner_interval)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences

                if self.prefs.is_router and self.prefs.is_router is True:
                    self.is_router.setChecked(True)

                tmp_r = 'Unset'
                if self.prefs.region:
                    tmp_r = self.prefs.region
                    print(f'tmp_r:{tmp_r}')
                count = 0
                self.region.clear()
                desc = meshtastic.radioconfig_pb2.RegionCode.DESCRIPTOR
                for k,v in desc.values_by_name.items():
                    self.region.addItem(k, v.number)
                    if v.number == tmp_r:
                        self.region.setCurrentIndex(count)
                    count = count + 1

                if self.prefs.debug_log_enabled and self.prefs.debug_log_enabled is True:
                    self.debug_log_enabled.setChecked(True)

                if self.prefs.serial_disabled and self.prefs.serial_disabled is True:
                    self.serial_disabled.setChecked(True)

                if self.prefs.auto_screen_carousel_secs:
                    self.auto_screen_carousel_secs.setText(f'{self.prefs.auto_screen_carousel_secs}')
                else:
                    self.auto_screen_carousel_secs.setText("0")

                if self.prefs.frequency_offset:
                    self.frequency_offset.setText(f'{self.prefs.frequency_offset}')
                else:
                    self.frequency_offset.setText("0")

                if self.prefs.hop_limit:
                    self.hop_limit.setText(f'{self.prefs.hop_limit}')
                else:
                    self.hop_limit.setText("0")

#                if self.prefs.ignore_incoming:
#                    self.ignore_incoming.setText(f'{self.prefs.ignore_incoming}')
#                else:
#                    self.ignore_incoming.setText("0")

                if self.prefs.is_lora_tx_disabled and self.prefs.is_lora_tx_disabled is True:
                    self.is_lora_tx_disabled.setChecked(True)

                if self.prefs.send_owner_interval:
                    self.send_owner_interval.setText(f'{self.prefs.send_owner_interval}')
                else:
                    self.send_owner_interval.setText("0")

        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'is_router', f'{self.is_router.isChecked()}')
                setPref(prefs, 'region', f'{self.region.currentData()}')
                setPref(prefs, 'debug_log_enabled', f'{self.debug_log_enabled.isChecked()}')
                setPref(prefs, 'serial_disabled', f'{self.serial_disabled.isChecked()}')
                setPref(prefs, 'auto_screen_carousel_secs', zero_if_blank(self.auto_screen_carousel_secs.text()))
                setPref(prefs, 'frequency_offset', zero_if_blank(self.frequency_offset.text()))
                setPref(prefs, 'hop_limit', zero_if_blank(self.hop_limit.text()))
                #setPref(prefs, 'ignore_incoming', zero_if_blank(self.ignore_incoming.text()))
                setPref(prefs, 'is_lora_tx_disabled', f'{self.is_lora_tx_disabled.isChecked()}')
                setPref(prefs, 'send_owner_interval', zero_if_blank(self.send_owner_interval.text()))
                self.interface.getNode(BROADCAST_ADDR).writeConfig()

        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()



"""class for the rotary encoder plugin settings"""


from PySide6 import QtCore
from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QDialogButtonBox, QLineEdit, QLabel, QComboBox

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
import meshtastic.radioconfig_pb2
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref
from meshtastic_flasher.util import zero_if_blank


class RotaryEncoderForm(QDialog):
    """serial plugin settings form"""

    def __init__(self, parent=None):
        """constructor"""
        super(RotaryEncoderForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Rotary Encoder Plugin Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets
        self.rotary1_about = QLabel("<a href='https://meshtastic.org/docs/software/settings/rotary-encoder-plugin'>More info</a>")
        self.rotary1_about.setOpenExternalLinks(True)
        self.rotary1_about.setTextFormat(QtCore.Qt.RichText)
        self.rotary1_about.setToolTip("Link shows more info about the settings for this plugin.")
        self.rotary1_enabled = QCheckBox()
        self.rotary1_enabled.setToolTip(self.parent.parent.description('rotary1_enabled'))
        self.rotary1_event_cw = QComboBox()
        self.rotary1_event_cw.setToolTip(self.parent.parent.description('rotary1_event_cw'))
        self.rotary1_event_cw.setMinimumContentsLength(17)
        self.rotary1_event_ccw = QComboBox()
        self.rotary1_event_ccw.setToolTip(self.parent.parent.description('rotary1_event_ccw'))
        self.rotary1_event_ccw.setMinimumContentsLength(17)
        self.rotary1_event_press = QComboBox()
        self.rotary1_event_press.setToolTip(self.parent.parent.description('rotary1_event_press'))
        self.rotary1_event_press.setMinimumContentsLength(17)
        self.rotary1_pin_a = QLineEdit()
        self.rotary1_pin_a.setToolTip(self.parent.parent.description('rotary1_pin_a'))
        self.rotary1_pin_b = QLineEdit()
        self.rotary1_pin_b.setToolTip(self.parent.parent.description('rotary1_pin_b'))
        self.rotary1_pin_press = QLineEdit()
        self.rotary1_pin_press.setToolTip(self.parent.parent.description('rotary1_pin_press'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow("Rotary Encoder Plugin", self.rotary1_about)
        form_layout.addRow(self.parent.parent.label("rotary1_enabled"), self.rotary1_enabled)
        form_layout.addRow(self.parent.parent.label("rotary1_event_cw"), self.rotary1_event_cw)
        form_layout.addRow(self.parent.parent.label("rotary1_event_ccw"), self.rotary1_event_ccw)
        form_layout.addRow(self.parent.parent.label("rotary1_event_press"), self.rotary1_event_press)
        form_layout.addRow(self.parent.parent.label("rotary1_pin_a"), self.rotary1_pin_a)
        form_layout.addRow(self.parent.parent.label("rotary1_pin_b"), self.rotary1_pin_b)
        form_layout.addRow(self.parent.parent.label("rotary1_pin_press"), self.rotary1_pin_press)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences

                if self.prefs.rotary1_enabled and self.prefs.rotary1_enabled is True:
                    self.rotary1_enabled.setChecked(True)

                temp = 0
                if self.prefs.rotary1_event_cw:
                    temp = int(self.prefs.rotary1_event_cw)
                print(f'temp:{temp}')
                self.rotary1_event_cw.clear()
                desc = meshtastic.radioconfig_pb2.InputEventChar.DESCRIPTOR
                for k,v in desc.values_by_name.items():
                    print(f'k:{k} v.number:{v.number}')
                    self.rotary1_event_cw.addItem(k, v.number)
                temp2 = self.rotary1_event_cw.findData(temp)
                if temp2 != -1:
                    self.rotary1_event_cw.setCurrentIndex(temp2)

                temp = 0
                if self.prefs.rotary1_event_ccw:
                    temp = int(self.prefs.rotary1_event_ccw)
                print(f'temp:{temp}')
                self.rotary1_event_ccw.clear()
                desc = meshtastic.radioconfig_pb2.InputEventChar.DESCRIPTOR
                for k,v in desc.values_by_name.items():
                    print(f'k:{k} v.number:{v.number}')
                    self.rotary1_event_ccw.addItem(k, v.number)
                temp2 = self.rotary1_event_ccw.findData(temp)
                if temp2 != -1:
                    self.rotary1_event_ccw.setCurrentIndex(temp2)

                temp = 0
                if self.prefs.rotary1_event_press:
                    temp = int(self.prefs.rotary1_event_press)
                print(f'temp:{temp}')
                self.rotary1_event_press.clear()
                desc = meshtastic.radioconfig_pb2.InputEventChar.DESCRIPTOR
                for k,v in desc.values_by_name.items():
                    print(f'k:{k} v.number:{v.number}')
                    self.rotary1_event_press.addItem(k, v.number)
                temp2 = self.rotary1_event_press.findData(temp)
                if temp2 != -1:
                    self.rotary1_event_press.setCurrentIndex(temp2)

                if self.prefs.rotary1_pin_a:
                    self.rotary1_pin_a.setText(f'{self.prefs.rotary1_pin_a}')
                else:
                    self.rotary1_pin_a.setText("0")

                if self.prefs.rotary1_pin_b:
                    self.rotary1_pin_b.setText(f'{self.prefs.rotary1_pin_b}')
                else:
                    self.rotary1_pin_b.setText("0")

                if self.prefs.rotary1_pin_press:
                    self.rotary1_pin_press.setText(f'{self.prefs.rotary1_pin_press}')
                else:
                    self.rotary1_pin_press.setText("0")


        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'rotary1_enabled', f'{self.rotary1_enabled.isChecked()}')
                setPref(prefs, 'rotary1_event_cw', f'{self.rotary1_event_cw.currentData()}')
                setPref(prefs, 'rotary1_event_ccw', f'{self.rotary1_event_ccw.currentData()}')
                setPref(prefs, 'rotary1_event_press', f'{self.rotary1_event_press.currentData()}')
                setPref(prefs, 'rotary1_pin_a', zero_if_blank(self.rotary1_pin_a.text()))
                setPref(prefs, 'rotary1_pin_b', zero_if_blank(self.rotary1_pin_b.text()))
                setPref(prefs, 'rotary1_pin_press', zero_if_blank(self.rotary1_pin_press.text()))
                self.interface.getNode(BROADCAST_ADDR).writeConfig()

        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()



"""class for the user settings"""


from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QLineEdit, QLabel, QComboBox, QDialogButtonBox

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
from meshtastic.__init__ import BROADCAST_ADDR


class UserForm(QDialog):
    """user settings form"""

    def __init__(self, parent=None):
        """constructor"""
        super(UserForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("User Settings")

        self.port = None
        self.interface = None

        # Create widgets
        self.device_id = QLabel()
        self.device_id.setToolTip(self.parent.description('device_id'))
        self.hw_model = QLabel()
        self.hw_model.setToolTip(self.parent.description('hw_model'))
        self.macaddr = QLabel()
        self.macaddr.setToolTip(self.parent.description('macaddr'))
        self.long_name = QLineEdit()
        self.long_name.setToolTip(self.parent.description('long_name'))
        self.long_name.setMaxLength(self.parent.max_size('long_name'))
        self.long_name.setFixedWidth(self.parent.max_size('long_name') * self.parent.pixel_mult)
        self.short_name = QLineEdit()
        self.short_name.setToolTip(self.parent.description('short_name'))
        self.short_name.setMaxLength(self.parent.max_size('short_name'))
        # TODO: should improve this a bit
        #self.short_name.setFixedWidth(self.parent.max_size('short_name') * self.parent.pixel_mult)
        self.is_licensed = QCheckBox()
        self.is_licensed.setToolTip(self.parent.description('is_licensed'))
        self.team = QComboBox()
        self.team.setMinimumContentsLength(17)
        self.team.setToolTip(self.parent.description('team'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.label('device_id'), self.device_id)
        form_layout.addRow(self.parent.label('hw_model'), self.hw_model)
        form_layout.addRow(self.parent.label('macaddr'), self.macaddr)
        form_layout.addRow(self.parent.label('long_name'), self.long_name)
        form_layout.addRow(self.parent.label('short_name'), self.short_name)
        form_layout.addRow(self.parent.label('is_licensed'), self.is_licensed)
        form_layout.addRow(self.parent.label('team'), self.team)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface:
                for n in self.interface.nodes.values():
                    if n['num'] == self.interface.myInfo.my_node_num:

                        if 'id' in n['user']:
                            self.device_id.setText(n['user']['id'])
                        else:
                            self.device_id.setText('')

                        if 'hwModel' in n['user']:
                            self.hw_model.setText(n['user']['hwModel'])
                        else:
                            self.hw_model.setText('')

                        if 'macaddr' in n['user']:
                            self.macaddr.setText(meshtastic.util.convert_mac_addr(n['user']['macaddr']))
                        else:
                            self.macaddr.setText('')

                        if 'longName' in n['user']:
                            self.long_name.setText(n['user']['longName'])
                        else:
                            self.long_name.setText('')

                        if 'shortName' in n['user']:
                            self.short_name.setText(n['user']['shortName'])
                        else:
                            self.short_name.setText('')

                        if 'isLicensed' in n['user']:
                            self.is_licensed.setChecked(True)

                        tmp_team = 'CLEAR'
                        if 'team' in n['user']:
                            tmp_team = n['user']['team']
                            print(f'tmp_team:{tmp_team}')
                        count = 0
                        self.team.clear()
                        desc = meshtastic.mesh_pb2.Team.DESCRIPTOR
                        for k,v in desc.values_by_name.items():
                            self.team.addItem(k, v.number)
                            if k == tmp_team:
                                self.team.setCurrentIndex(count)
                            count = count + 1

        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing values to device")
                self.interface.getNode(BROADCAST_ADDR).setOwner(long_name=self.long_name.text(), short_name=self.short_name.text(), is_licensed=self.is_licensed.isChecked(), team=self.team.currentData())
        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()



"""class for the Wifi and MQTT settings"""


from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QLineEdit, QDialogButtonBox

import meshtastic.serial_interface
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref


class Wifi_and_MQTT_Form(QDialog):
    """wifi and mqtt form"""

    def __init__(self, parent=None):
        """constructor"""
        super(Wifi_and_MQTT_Form, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Wifi & MQTT Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets

        # WiFi
        self.wifi_ap_mode = QCheckBox()
        self.wifi_ap_mode.setToolTip(self.parent.description('wifi_ap_mode'))
        self.wifi_ssid = QLineEdit()
        self.wifi_ssid.setToolTip(self.parent.description('wifi_ssid'))
        self.wifi_ssid.setMaxLength(self.parent.max_size('wifi_ssid'))
        self.wifi_ssid.setFixedWidth(self.parent.max_size('wifi_ssid') * self.parent.pixel_mult)
        self.wifi_password = QLineEdit()
        self.wifi_password.setToolTip(self.parent.description('wifi_password'))
        self.wifi_password.setMaxLength(self.parent.max_size('wifi_password'))
        self.wifi_password.setFixedWidth(self.parent.max_size('wifi_password') * self.parent.pixel_mult)

        # MQTT
        self.mqtt_disabled = QCheckBox()
        self.mqtt_disabled.setToolTip(self.parent.description('mqtt_disabled'))
        self.mqtt_server = QLineEdit()
        self.mqtt_server.setToolTip(self.parent.description('mqtt_server'))
        self.mqtt_server.setMaxLength(self.parent.max_size('mqtt_server'))
        self.mqtt_server.setFixedWidth(self.parent.max_size('mqtt_server') * self.parent.pixel_mult)
        self.mqtt_username = QLineEdit()
        self.mqtt_username.setToolTip(self.parent.description('mqtt_username'))
        self.mqtt_username.setMaxLength(self.parent.max_size('mqtt_username'))
        self.mqtt_username.setFixedWidth(self.parent.max_size('mqtt_username') * self.parent.pixel_mult)
        self.mqtt_password = QLineEdit()
        self.mqtt_password.setToolTip(self.parent.description('mqtt_password'))
        self.mqtt_password.setMaxLength(self.parent.max_size('mqtt_password'))
        self.mqtt_password.setFixedWidth(self.parent.max_size('mqtt_password') * self.parent.pixel_mult)
        self.mqtt_encryption_enabled = QCheckBox()
        self.mqtt_encryption_enabled.setToolTip(self.parent.description('mqtt_encryption_enabled'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)


        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.label("wifi_ap_mode"), self.wifi_ap_mode)
        form_layout.addRow(self.parent.label("wifi_ssid"), self.wifi_ssid)
        form_layout.addRow(self.parent.label("wifi_password"), self.wifi_password)
        form_layout.addRow(self.parent.label("mqtt_disabled"), self.mqtt_disabled)
        form_layout.addRow(self.parent.label("mqtt_server"), self.mqtt_server)
        form_layout.addRow(self.parent.label("mqtt_username"), self.mqtt_username)
        form_layout.addRow(self.parent.label("mqtt_password"), self.mqtt_password)
        form_layout.addRow(self.parent.label("mqtt_encryption_enabled"), self.mqtt_encryption_enabled)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        print(f'port:{port}')
        if self.port:
            print(f'using port:{self.port}')
            self.get_prefs()
            print(f'prefs:{self.prefs}')

            if self.prefs.wifi_ap_mode and self.prefs.wifi_ap_mode is True:
                self.wifi_ap_mode.setChecked(True)

            if self.prefs.wifi_ssid:
                self.wifi_ssid.setText(self.prefs.wifi_ssid)
            else:
                self.wifi_ssid.setText("")

            if self.prefs.wifi_password:
                self.wifi_password.setText(self.prefs.wifi_password)
            else:
                self.wifi_password.setText("")

            if self.prefs.mqtt_disabled and self.prefs.mqtt_disabled is True:
                self.mqtt_disabled.setChecked(True)

            if self.prefs.mqtt_server:
                self.mqtt_server.setText(self.prefs.mqtt_server)
            else:
                self.mqtt_server.setText("")

            if self.prefs.mqtt_username:
                self.mqtt_username.setText(self.prefs.mqtt_username)
            else:
                self.mqtt_username.setText("")

            if self.prefs.mqtt_password:
                self.mqtt_password.setText(self.prefs.mqtt_password)
            else:
                self.mqtt_password.setText("")

            if self.prefs.mqtt_encryption_enabled and self.prefs.mqtt_encryption_enabled is True:
                self.mqtt_encryption_enabled.setChecked(True)

            self.show()


    def get_prefs(self):
        """Get preferences from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
        except Exception as e:
            print(f'Exception:{e}')


    def write_prefs(self):
        """Write preferences to device"""
        try:
            if self.interface:
                print("Writing modified preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'wifi_ap_mode', f'{self.wifi_ap_mode.isChecked()}' )
                setPref(prefs, 'wifi_ssid', self.wifi_ssid.text())
                setPref(prefs, 'wifi_password', self.wifi_password.text())
                setPref(prefs, 'mqtt_disabled', f'{self.mqtt_disabled.isChecked()}' )
                setPref(prefs, 'mqtt_server', self.mqtt_server.text())
                setPref(prefs, 'mqtt_username', self.mqtt_username.text())
                setPref(prefs, 'mqtt_password', self.mqtt_password.text())
                setPref(prefs, 'mqtt_encryption_enabled', f'{self.mqtt_encryption_enabled.isChecked()}' )
                self.interface.getNode(BROADCAST_ADDR).writeConfig()
        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_prefs()
